#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stm8l15x_adc.h>
#include <stm8l15x_tim1.h>
#include <stm8l15x_clk.h>
#include "globdef.h"
#include "ADCdriverFFT.h"
#include "mono_lcd.h"
#include "mallocR.h"


#define F_SAMPLING 1000     //1kHz sampling
#define AVERAGE_LENGTH 10   //how many samples to average in one set
#define ADC_BUFFER_SIZE (AVERAGE_LENGTH * 20) //20 averaged set of samples


//collecting data from ADC into buffer
static void ADCCollectDataAvrg(unsigned int * ADCBuffer)
{
    unsigned int i;
    unsigned int divider;
    
    //timer trigger init
    divider = F_CPU / F_SAMPLING;
    //enable clock for timer 1
    CLK_PeripheralClockConfig(CLK_Peripheral_TIM1, ENABLE);
    
    //set sampling frequency by timer
    TIM1_TimeBaseInit(0,
                       TIM1_CounterMode_Down,
                       divider,
                       0);
                       
    //TRGO enable on update event
    TIM1_SelectOutputTrigger(TIM1_TRGOSource_Update);
    TIM1_ARRPreloadConfig(ENABLE);
    //update timer
    TIM1_GenerateEvent(TIM1_EventSource_Update);
    
    //disable DMA for ADC
    ADC_DMACmd(ADC1, DISABLE);
    //disable ADC interrupts
    ADC_ITConfig(ADC1, ADC_IT_EOC | ADC_IT_AWD | ADC_IT_OVER, DISABLE);
    //init and enable ADC (one channel AIN_24)
    ADC_Init(ADC1,
              ADC_ConversionMode_Single,
              ADC_Resolution_12Bit,
              ADC_Prescaler_1);
    ADC_ChannelCmd(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SchmittTriggerConfig(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SamplingTimeConfig(ADC1,
                            ADC_Group_FastChannels, //channel 24  is in fast group
                            ADC_SamplingTime_16Cycles);
    ADC_ExternalTrigConfig(ADC1,
                            ADC_ExtEventSelection_Trigger2, //for timer1 event is selected trigger2
                            ADC_ExtTRGSensitivity_Rising);
                              
    //start timer - periodic trigger 
    TIM1_Cmd(ENABLE);
    
    //sample data
    i=0;
    while (i<ADC_BUFFER_SIZE)
    {
      //wait for EOC
      while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
      //collect ADC value
      ADCBuffer [i] = ADC_GetConversionValue(ADC1);
      i++;
    }

    //stop data sampling
    TIM1_Cmd(DISABLE);  //stop timer - trigger
}//ADCCollectData
//perform averaging on whole collected ADC data (floating average)
void Average(unsigned int * ADCBuffer)
{
  unsigned int i,j;
  unsigned long AverageSum;

  //averaging
  for (i=0; i<(ADC_BUFFER_SIZE-AVERAGE_LENGTH); i++)
  {
    AverageSum = 0;
    //sum set of samples (can be optimized)
    for(j=0; j<AVERAGE_LENGTH; j++)
      AverageSum += ADCBuffer[j+i];
    
    //write back the averaged sample
    ADCBuffer[i]= AverageSum/AVERAGE_LENGTH;
  }
}//Average
//get min and max value difference in ADC data buffer (to compare signal stability)
static unsigned int GetMinMaxAvrg(unsigned int * Buffer, unsigned int buffersize)
{
  unsigned int i,min,max;
  
  min=0xFFFF;
  max=0;
  for (i=0; i<buffersize; i++)
  {
    if (Buffer[i]<min)
      min = Buffer[i];
    if (Buffer[i]>max)
      max = Buffer[i];
  }
  return(max-min);
}
//main routine for Averaging method testing
void TestADCAveraging(void)
{
  unsigned int * ADCBuffer;
  volatile unsigned int diffOrig;
  volatile unsigned int diffAvrg;
  
  //allocate buffer space
  if ((ADCBuffer = malloc(ADC_BUFFER_SIZE * sizeof(ADCBuffer[0]))) == NULL)
    trap();
  
  //collect data
  ADCCollectDataAvrg(ADCBuffer);
  diffOrig = GetMinMaxAvrg(ADCBuffer, ADC_BUFFER_SIZE-AVERAGE_LENGTH);
 
  //filter data
  Average(ADCBuffer);
  diffAvrg = GetMinMaxAvrg(ADCBuffer, ADC_BUFFER_SIZE-AVERAGE_LENGTH);
  
  LCD_PrintString(LCD_LINE1, DISABLE, DISABLE, "DiffOrig= ");
  LCD_PrintDec2(diffOrig/100);
  LCD_PrintDec2(diffOrig%100);
    
  LCD_PrintString(LCD_LINE2, DISABLE, DISABLE, "DiffAvrg= ");
  LCD_PrintDec2(diffAvrg/100);
  LCD_PrintDec2(diffAvrg%100);

  //unallocate buffer
  free(ADCBuffer);
}//TestADCAveraging
