#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stm8l15x_adc.h>
#include <stm8l15x_tim1.h>
#include <stm8l15x_clk.h>
#include "globdef.h"
#include "mono_lcd.h"
#include "mallocR.h"

#define F_SAMPLING 1000   //1kHz sampling
#define F_MODULATION 100  //white noise simulation (100Hz signal)
#define MOD_PERIODS 20    //number of modulation periods
#define ADC_BUFFER_SIZE (F_SAMPLING/F_MODULATION * 20) //buffer for modulation periods


//collecting data from ADC into buffer
static void ADCCollectDataWhiteNoise(unsigned int * ADCBuffer)
{
    unsigned int i;
    unsigned int divider;
    
    //timer trigger init
    divider = F_CPU / F_SAMPLING;
    //enable clock for timer 1
    CLK_PeripheralClockConfig(CLK_Peripheral_TIM1, ENABLE);
    
    //set sampling frequency by timer
    TIM1_TimeBaseInit(0,
                       TIM1_CounterMode_Down,
                       divider,
                       0);
                       
    //TRGO enable on update event
    TIM1_SelectOutputTrigger(TIM1_TRGOSource_Update);
    TIM1_ARRPreloadConfig(ENABLE);
    //update timer
    TIM1_GenerateEvent(TIM1_EventSource_Update);
    
    //disable ADC interrupts
    ADC_ITConfig(ADC1, ADC_IT_EOC | ADC_IT_AWD | ADC_IT_OVER, DISABLE);
    //disable DMA for ADC
    ADC_DMACmd(ADC1, DISABLE);
    //init and enable ADC (one channel AIN_24)
    ADC_Init(ADC1,
              ADC_ConversionMode_Single,
              ADC_Resolution_12Bit,
              ADC_Prescaler_1);
    ADC_ChannelCmd(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SchmittTriggerConfig(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SamplingTimeConfig(ADC1,
                            ADC_Group_FastChannels, //channel 24  is in fast group
                            ADC_SamplingTime_16Cycles);
    ADC_ExternalTrigConfig(ADC1,
                            ADC_ExtEventSelection_Trigger2, //for timer1 event is selected trigger2
                            ADC_ExtTRGSensitivity_Rising);
                              
    //start timer - periodic trigger 
    TIM1_Cmd(ENABLE);
    
    //sample data
    i=0;
    while (i<ADC_BUFFER_SIZE)
    {
      //wait for EOC
      while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
      //collect ADC value
      ADCBuffer [i] = ADC_GetConversionValue(ADC1);
      i++;
    }

    //stop data sampling
    TIM1_Cmd(DISABLE);  //stop timer - trigger
}//ADCCollectData
//perform averaging on whole collected ADC data 
//white noise into signal must be added externally
static double AverageWhiteNoise(unsigned int * ADCBuffer)
{
  //Averaging implementation to show white noise spreading (white noise must be connected) 
  unsigned int i;
  double AverageSum;

  //averaging
  AverageSum = 0;
  for (i=0; i<(ADC_BUFFER_SIZE); i++)
  {
    //sum set of all samples
    AverageSum += ADCBuffer[i];
  }
  return (AverageSum/ADC_BUFFER_SIZE);
}//AverageWhiteNoise
//main routine for white noise adding method testing
void TestADCWhiteNoise(void)
{
  unsigned int * ADCBuffer;
  double WhiteNoiseResult;
  
  //allocate buffer space
  if ((ADCBuffer = malloc(ADC_BUFFER_SIZE * sizeof(ADCBuffer[0]))) == NULL)
    trap();
  
  //collect data
  ADCCollectDataWhiteNoise(ADCBuffer);
  //filter data from white noise
  WhiteNoiseResult = AverageWhiteNoise(ADCBuffer);
  
  LCD_PrintString(LCD_LINE1, DISABLE, DISABLE, "WhiteNoiseSpread");
  LCD_PrintString(LCD_LINE2, DISABLE, DISABLE, "V= ");
  LCD_PrintDec2(WhiteNoiseResult/100);
  LCD_PrintDec2((u32)WhiteNoiseResult%100);
  LCD_PrintChar('.');
  LCD_PrintDec2((u32)(WhiteNoiseResult*100)%100);

  //unallocate buffer
  free(ADCBuffer);
}//TestADCWhiteNoise
