//functions for ADC management for FFT analysis
#include <stm8l15x.h>
#include <stm8l15x_gpio.h>
#include <stm8l15x_adc.h>
#include <stm8l15x_tim1.h>
#include <stm8l15x_clk.h>
#include "globdef.h"
#include "globaldefFFT.h"
#include "ADCdriverFFT.h"


PParam ADCpparam;
ADC_Channel_TypeDef ChannelIndex[27]=
{
ADC_Channel_0         ,
ADC_Channel_1         ,
ADC_Channel_2         ,
ADC_Channel_3         ,
ADC_Channel_4         ,
ADC_Channel_5         ,
ADC_Channel_6         ,
ADC_Channel_7         ,
ADC_Channel_8         ,
ADC_Channel_9         ,
ADC_Channel_10        ,
ADC_Channel_11        ,
ADC_Channel_12        ,
ADC_Channel_13        ,
ADC_Channel_14        ,
ADC_Channel_15        ,
ADC_Channel_16        ,
ADC_Channel_17        ,
ADC_Channel_18        ,
ADC_Channel_19        ,
ADC_Channel_20        ,
ADC_Channel_21        ,
ADC_Channel_22        ,
ADC_Channel_23        ,
ADC_Channel_24        ,
ADC_Channel_Vrefint   ,
ADC_Channel_TempSensor
};



//setting of ADC for FFT data collection (timer trigger)
void InitADCTimerTrigger(PParam pparam, unsigned int divider)
{
    unsigned char channels;
    
    //remember global param
    ADCpparam = pparam;
    ADCpparam->ConvNumber = 0;
    ADCpparam->EndOfAllConversions = 1;

    //enable clock for timer 1
    CLK_PeripheralClockConfig(CLK_Peripheral_TIM1, ENABLE);

    //set sampling frequency by timer
    TIM1_TimeBaseInit(0,
                       TIM1_CounterMode_Down,
                       divider,
                       0);

    //TRGO enable on update event
    TIM1_SelectOutputTrigger(TIM1_TRGOSource_Update);
    TIM1_ARRPreloadConfig(ENABLE);
    //update timer
    //TIM1_GenerateEvent(TIM1_EventSource_Update);


    //disable Schmitt triggers on measured channels
    channels = ADCpparam->StartInputChannel;
    while (channels != (ADCpparam->StartInputChannel + ADCpparam->InputChannelCount))
      ADC_SchmittTriggerConfig(ADC1, ChannelIndex[channels++], DISABLE);

    //disable ADC interrupts
    ADC_ITConfig(ADC1, ADC_IT_EOC | ADC_IT_AWD | ADC_IT_OVER, DISABLE);
    //disable DMA for ADC
    ADC_DMACmd(ADC1, DISABLE);
    //init and enable ADC (one channel)
    ADC_Init(ADC1,
              ADC_ConversionMode_Single,
              ADC_Resolution_12Bit,
              ADC_Prescaler_1);
    //disable all channels
    for (channels = 0; channels < sizeof(ChannelIndex)/sizeof(ChannelIndex[0]); channels++)
      ADC_ChannelCmd(ADC1, ChannelIndex[channels], DISABLE);
    //enable one channel only               
    ADC_ChannelCmd(ADC1,
                    ChannelIndex[ADCpparam->StartInputChannel],
                    ENABLE);
    ADC_SchmittTriggerConfig(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SamplingTimeConfig(ADC1,
                            ADC_Group_FastChannels, //fast group
                            ADC_SamplingTime_16Cycles);
    ADC_SamplingTimeConfig(ADC1,
                            ADC_Group_SlowChannels, //slow group
                            ADC_SamplingTime_16Cycles);
    ADC_ExternalTrigConfig(ADC1,
                            ADC_ExtEventSelection_Trigger2, //for timer1 event is selected trigger2
                            ADC_ExtTRGSensitivity_Rising);
    //clear end of conversion bit
    ADC_ClearFlag(ADC1, ADC_FLAG_EOC | ADC_FLAG_OVER | ADC_FLAG_AWD);


    //enable ADC interrupts on EOC
    ADC_ITConfig(ADC1, ADC_IT_EOC, ENABLE);
    ADCpparam->EndOfAllConversions = 0;
}
//starting data collection
void ADCStart(void)
{
  enableInterrupts();
  //start timer - periodic trigger 
  TIM1_Cmd(ENABLE);
}
//interrupt service routine for ADC - data collection for FFT into buffer
#ifdef _COSMIC_
@far @interrupt void ADCInterrupt(void)
#elif defined(_RAISONANCE_) /* _RAISONANCE_ */
void ADCInterrupt(void) interrupt 18
#else /* _IAR_ */
#pragma vector=20
__interrupt void ADCInterrupt(void)
#endif /* _COSMIC_ */

{
    //channel counter
    static unsigned char channel;

    //if no operation is needed - return
    if (ADCpparam->EndOfAllConversions)
    {
      ADC_ClearFlag(ADC1, ADC_FLAG_EOC|ADC_FLAG_OVER|ADC_FLAG_AWD);
      return;
    }

    //init channel counter at first conversion
    if(!ADCpparam->ConvNumber)
      channel = ADCpparam->StartInputChannel;

    //collect ADC value
    ADCpparam->AddrADCDataStart[ADCpparam->ConvNumber] = ADC_GetConversionValue(ADC1)<<3;//convert to signed int
    ADCpparam->ConvNumber++; //increment counter

    // increment ADC channel
    if (channel == (ADCpparam->StartInputChannel + ADCpparam->InputChannelCount -1))
    {
      ADC_ChannelCmd(ADC1, ChannelIndex[channel], DISABLE);
      channel = ADCpparam->StartInputChannel;      
      ADC_ChannelCmd(ADC1, ChannelIndex[channel], ENABLE);
    }
    else
    {
      ADC_ChannelCmd(ADC1, ChannelIndex[channel], DISABLE);
      channel++;
      ADC_ChannelCmd(ADC1, ChannelIndex[channel], ENABLE);
    }

    //test to stop data sampling
    if (ADCpparam->ConvNumber == ADCpparam->NumOfConversions)
    {
      ADC_ITConfig(ADC1, ADC_IT_EOC, DISABLE);//disable ADC interrupt
      TIM1_Cmd(DISABLE);                  //stop timer - trigger
      ADCpparam->EndOfAllConversions = 1; //signalize end of conversions
      ADC_ChannelCmd(ADC1, ChannelIndex[channel], DISABLE);//disable current channel
    }
  return;
}
