#include <stm8l15x_spi.h>
#include <stm8l15x_gpio.h>
#include <stm8l15x_clk.h>
#include <stm8l15x_adc.h>
#include <stm8l15x.h>
#include <stdio.h>
#include <stdlib.h>
#include "globdef.h"
#include "mono_lcd.h"
#include "ADCAveraging.h"
#include "ADCWhiteNoise.h"
#include "Filter50Hz.h"
#include "FFT.h"
#include "ADCCalibration.h"
#include "ADCdriverFFT.h"

#define DELAY_BUTTON 400000l
#ifdef _IAR_
#define RollSpeed 10000
#else /* _COSMIC_ and _RAISONANCE_ */
#define RollSpeed 60000
#endif /*_IAR_*/
//end of memory definition for linker
#ifdef _RAISONANCE_
#else /* COSMIC */
//extern unsigned char _endmem @0x7ff;
#endif

static void delay(__IO u32 cycles)
{
  while (cycles--);
}


 // MAIN *********************************
//main project routine
void main(void)
{
  //set full speed clock
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
  
  //enable clock for ADC
  CLK_PeripheralClockConfig(CLK_Peripheral_ADC1, ENABLE);
  //enable ADC
  ADC_Cmd(ADC1, ENABLE);
  
  //init LCD
  InitLCDDisplay();
  
  //invitation screen
  LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "ADC STM8L demo");  
  LCD_RollString(LCD_LINE2, "              Please set JP10 to MC position and connect signal to BNC connector!", RollSpeed);  
  LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "press Joystick");  
  while(!JOY_SEL);
  
  //main program loop
  do
  {  
    //print DEMO information
    LCD_Clear();

    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, " Precision demo ");  
    LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "   STM8L15x   ");  
    delay(DELAY_BUTTON);

    //----------------------------------------------------------
    //"Averaging" test
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "Averaging");
    delay(DELAY_BUTTON);
    do
    {  
      TestADCAveraging();
    }
    while(!BUTTON_LOW);
  
    //----------------------------------------------------------
    //"White Noise adding" test
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "WhiteNoise");
    delay(DELAY_BUTTON);
    do
    {   
      TestADCWhiteNoise();
    }    
    while(!BUTTON_LOW);
  
    //----------------------------------------------------------
    //"Digital filter" test (removing 50Hz disturbance)
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "Filter50Hz");
    delay(DELAY_BUTTON);
    do
    {  
      TestADCDigitalFilter50Hz();
    }    
    while(!BUTTON_LOW);
    
    //----------------------------------------------------------
    //"Linear calibration" test (gain and offsett ADC calibration)
    //measure real value (by precise external instrument) and ADC value
    //enter real value by joystick buttons (up/down = increase/decrease digit, left/right = move to another digit, select = enter value)
    //finish entering = key button
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "Calibration");
    delay(DELAY_BUTTON);  
    do
    {  
      TestADCCalibration();  
    }    
    while(!BUTTON_LOW);
  
    //----------------------------------------------------------
    //"Fast Fourrier Transformation" test 
    //measuring 3-phase AC signal RMS values: active powers, voltages, currents = 3-phase electricity meter
    //connect 6 signals to AIN0 - AIN5 (representing 3-phase signals: U1,I1, U2,I2, U3,I3)
    //signals have to be 50Hz sinus signals - synchronous as 3-phase power system
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "FFT 16 points");
    LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "see debug data");
    delay(DELAY_BUTTON);  
    do
    {  
      LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "calculating...");
      //next line commented due to 16kB free compiler limitation (uncomment for full version or 32kB free version of compiler)
      TestADCFFT();
      LCD_PrintString(LCD_LINE2, ENABLE, DISABLE, "done");
      delay(DELAY_BUTTON);  
    }  
    while(!BUTTON_LOW);
  
    //----------------------------------------------------------
    //print that demo finished
    LCD_Clear();
    LCD_PrintString(LCD_LINE1, ENABLE, DISABLE, "End of demo");
    delay(DELAY_BUTTON);  
  }  
  while(1);//repeat tests in endless loop
}/*main*/
 // MAIN *********************************
