/**
  ******************************************************************************
  * @file mono_lcd.h
  * @brief This file contains LCD functions prototypes.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 04/10/2009
  ******************************************************************************
  *
  */

#ifndef __MONO_LCD_H
#define __MONO_LCD_H

// Includes ------------------------------------------------------------------
#include "stm8l15x.h"

// Exported constants --------------------------------------------------------

// LCD Chip Select I/O definition 
#define LCD_CS_PORT (GPIOB)
#define LCD_CS_PIN  (GPIO_Pin_4)

// LCD managed as 2 Lines of 15 characters (2Lines * 15Char) 
#define LCD_LINE_MAX_CHAR ((u8)15)
#define LCD_LINE1         ((u8)0x80)
#define LCD_LINE2         ((u8)0x90)

// LCD Commands 
#define COMMAND_TYPE       ((u8)0xF8)
#define DATA_TYPE          ((u8)0xFA)
#define SET_TEXT_MODE      ((u8)0x30) /* 8-Bits Interface, Normal inst., Text mode */
#define SET_EXTENDED_INST  ((u8)0x34)
#define SET_GRAPHIC_MODE   ((u8)0x36) /* 8-Bits Interface, Extended inst., Graphic mode */
#define DISPLAY_ON         ((u8)0x0C) /* Cursor and blink off */
#define DISPLAY_OFF        ((u8)0x08)
#define DISPLAY_CLR        ((u8)0x01)
#define ENTRY_MODE_SET_INC ((u8)0x06)

// Exported macro ------------------------------------------------------------

// Exported functions --------------------------------------------------------
void LCD_ReadStatus(void);
void LCD_ChipSelect(FunctionalState NewState);
void LCD_SendByte(u8 DataType, u8 DataToSend);
void LCD_SendBuffer(u8 *pTxBuffer, u8 NumByte);
void LCD_Init(void);
void InitLCDDisplay(void);
void LCD_Clear(void);
void LCD_SetTextMode(void);
void LCD_SetGraphicMode(void);
void LCD_ClearLine(u8 Line);
void LCD_SetCursorPos(u8 Line, u8 Offset);
void LCD_PrintChar(u8 Ascii);
void LCD_PrintString(u8 Line, FunctionalState AutoComplete, FunctionalState Append, u8 *ptr);
void LCD_PrintMsg(u8 *ptr);
void LCD_Print(u8 *ptr);
void LCD_PrintDec1(u8 Number);
void LCD_PrintDec2(u8 Number);
void LCD_PrintDec3(u16 Number);
void LCD_PrintDec4(u16 Number);
void LCD_PrintHex1(u8 Number);
void LCD_PrintHex2(u8 Number);
void LCD_PrintHex3(u16 Number);
void LCD_PrintBin2(u8 Number);
void LCD_PrintBin4(u8 Number);
void LCD_DisplayCGRAM0(u8 address, u8 *ptrTable);
void LCD_DisplayCGRAM1(u8 address, u8 *ptrTable);
void LCD_DisplayLogo(u8 address);
void LCD_RollString(u8 Line, u8 *ptr, u16 speed);

#endif /* __MONO_LCD_H */
